@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -85,7 +85,7 @@ public final class FolderActionFactory implements AJAXActionServiceFactory {
     }
 
     private Map<String, AJAXActionService> initActions() {
-        final Map<String, AJAXActionService> tmp = new HashMap<String, AJAXActionService>();
+        final Map<String, AJAXActionService> tmp = new HashMap<String, AJAXActionService>(12);
         tmp.put(RootAction.ACTION, new RootAction());
         tmp.put(ListAction.ACTION, new ListAction());
         tmp.put(GetAction.ACTION, new GetAction());
@@ -95,6 +95,8 @@ public final class FolderActionFactory implements AJAXActionServiceFactory {
         tmp.put(PathAction.ACTION, new PathAction());
         tmp.put(ClearAction.ACTION, new ClearAction());
         tmp.put(UpdatesAction.ACTION, new UpdatesAction());
+        tmp.put(VisibleFoldersAction.ACTION, new VisibleFoldersAction());
+        tmp.put(SubscribeAction.ACTION, new SubscribeAction());
         return Collections.unmodifiableMap(tmp);
     }
 
