@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,19 +49,29 @@
 
 package com.openexchange.folder.json.writer;
 
-import gnu.trove.TIntIntHashMap;
-import gnu.trove.TIntObjectHashMap;
+import gnu.trove.ConcurrentTIntObjectHashMap;
+import gnu.trove.list.TIntList;
+import gnu.trove.list.array.TIntArrayList;
+import gnu.trove.map.TIntObjectMap;
+import gnu.trove.map.hash.TIntIntHashMap;
+import gnu.trove.map.hash.TIntObjectHashMap;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
+import java.util.List;
+import java.util.Locale;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 import com.openexchange.ajax.customizer.folder.AdditionalFolderField;
 import com.openexchange.ajax.customizer.folder.AdditionalFolderFieldList;
+import com.openexchange.ajax.customizer.folder.BulkFolderField;
 import com.openexchange.folder.json.FolderField;
+import com.openexchange.folder.json.FolderFieldRegistry;
 import com.openexchange.folderstorage.ContentType;
 import com.openexchange.folderstorage.FolderException;
 import com.openexchange.folderstorage.FolderExceptionErrorMessage;
+import com.openexchange.folderstorage.FolderProperty;
 import com.openexchange.folderstorage.Permission;
 import com.openexchange.folderstorage.Type;
 import com.openexchange.folderstorage.UserizedFolder;
@@ -75,7 +85,18 @@ import com.openexchange.tools.session.ServerSession;
  */
 public final class FolderWriter {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(FolderWriter.class);
+    /**
+     * The logger constant.
+     */
+    protected static final org.apache.commons.logging.Log LOG =
+        com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(FolderWriter.class));
+
+    protected static final boolean WARN = LOG.isWarnEnabled();
+
+    /**
+     * The default locale: en_US.
+     */
+    public static final Locale DEFAULT_LOCALE = Locale.US;
 
     /**
      * Initializes a new {@link FolderWriter}.
@@ -141,10 +162,18 @@ public final class FolderWriter {
 
     private static interface FolderFieldWriter {
 
+        /**
+         * Writes associated field's value to given JSON value.
+         * 
+         * @param jsonValue The JSON value
+         * @param folder The folder
+         * @throws JSONException If a JSON error occurs
+         * @throws NecessaryValueMissingException If a necessary value is missing; such as identifier
+         */
         void writeField(JSONValuePutter jsonValue, UserizedFolder folder) throws JSONException;
     }
 
-    private static final FolderFieldWriter UNKNOWN_FIELD_FFW = new FolderFieldWriter() {
+    protected static final FolderFieldWriter UNKNOWN_FIELD_FFW = new FolderFieldWriter() {
 
         public void writeField(final JSONValuePutter jsonValue, final UserizedFolder folder) throws JSONException {
             jsonValue.put("unknown_field", JSONObject.NULL);
@@ -161,6 +190,9 @@ public final class FolderWriter {
 
             public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
                 final String id = folder.getID();
+                if (null == id) {
+                    throw new NecessaryValueMissingException("Missing folder identifier.");
+                }
                 jsonPutter.put(FolderField.ID.getName(), id);
             }
         });
@@ -209,7 +241,8 @@ public final class FolderWriter {
         m.put(FolderField.FOLDER_NAME.getColumn(), new FolderFieldWriter() {
 
             public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
-                final String name = folder.getLocalizedName(folder.getLocale());
+                final Locale locale = folder.getLocale();
+                final String name = folder.getLocalizedName(locale == null ? DEFAULT_LOCALE : locale);
                 jsonPutter.put(FolderField.FOLDER_NAME.getName(), name);
             }
         });
@@ -231,7 +264,12 @@ public final class FolderWriter {
 
             public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
                 final String[] obj = folder.getSubfolderIDs();
-                jsonPutter.put(FolderField.SUBFOLDERS.getName(), null == obj ? JSONObject.NULL : Boolean.valueOf(obj.length > 0));
+                if (null == obj) {
+                    LOG.warn("Got null as subfolders for folder " + folder.getID() + ". Marking this folder to hold subfolders...");
+                    jsonPutter.put(FolderField.SUBFOLDERS.getName(), Boolean.TRUE);
+                } else {
+                    jsonPutter.put(FolderField.SUBFOLDERS.getName(), Boolean.valueOf(obj.length > 0));
+                }
             }
         });
         m.put(FolderField.OWN_RIGHTS.getColumn(), new FolderFieldWriter() {
@@ -240,12 +278,14 @@ public final class FolderWriter {
                 final int bits = folder.getBits();
                 if (bits < 0) {
                     final Permission obj = folder.getOwnPermission();
-                    jsonPutter.put(FolderField.OWN_RIGHTS.getName(), null == obj ? JSONObject.NULL : Integer.valueOf(createPermissionBits(
-                        obj.getFolderPermission(),
-                        obj.getReadPermission(),
-                        obj.getWritePermission(),
-                        obj.getDeletePermission(),
-                        obj.isAdmin()))); 
+                    jsonPutter.put(
+                        FolderField.OWN_RIGHTS.getName(),
+                        null == obj ? JSONObject.NULL : Integer.valueOf(createPermissionBits(
+                            obj.getFolderPermission(),
+                            obj.getReadPermission(),
+                            obj.getWritePermission(),
+                            obj.getDeletePermission(),
+                            obj.isAdmin())));
                 } else {
                     jsonPutter.put(FolderField.OWN_RIGHTS.getName(), Integer.valueOf(bits));
                 }
@@ -370,13 +410,11 @@ public final class FolderWriter {
     public static JSONArray writeSingle2Array(final int[] fields, final UserizedFolder folder) throws FolderException {
         final int[] cols = null == fields ? ALL_FIELDS : fields;
         final FolderFieldWriter[] ffws = new FolderFieldWriter[cols.length];
+        final TIntObjectMap<com.openexchange.folderstorage.FolderField> fieldSet = FolderFieldRegistry.getInstance().getFields();
         for (int i = 0; i < ffws.length; i++) {
             FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(cols[i]);
             if (null == ffw) {
-                if (LOG.isWarnEnabled()) {
-                    LOG.warn("Unknown field: " + cols[i], new Throwable());
-                }
-                ffw = UNKNOWN_FIELD_FFW;
+                ffw = getPropertyByField(cols[i], fieldSet);
             }
             ffws[i] = ffw;
         }
@@ -389,6 +427,8 @@ public final class FolderWriter {
             return jsonArray;
         } catch (final JSONException e) {
             throw FolderExceptionErrorMessage.JSON_ERROR.create(e, e.getMessage());
+        } catch (final NecessaryValueMissingException e) {
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
 
@@ -403,12 +443,14 @@ public final class FolderWriter {
     public static JSONArray writeMultiple2Array(final int[] fields, final UserizedFolder[] folders, final ServerSession serverSession, final AdditionalFolderFieldList additionalFolderFieldList) throws FolderException {
         final int[] cols = null == fields ? ALL_FIELDS : fields;
         final FolderFieldWriter[] ffws = new FolderFieldWriter[cols.length];
+        final TIntObjectMap<com.openexchange.folderstorage.FolderField> fieldSet = FolderFieldRegistry.getInstance().getFields();
         for (int i = 0; i < ffws.length; i++) {
             final int curCol = cols[i];
             FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(curCol);
             if (null == ffw) {
                 if (additionalFolderFieldList.knows(curCol)) {
-                    final AdditionalFolderField additionalFolderField = additionalFolderFieldList.get(curCol);
+                    final AdditionalFolderField aff = new BulkFolderField(additionalFolderFieldList.get(curCol));
+                    aff.getValues(turnIntoFolderObjects(folders), serverSession);
                     ffw = new FolderFieldWriter() {
 
                         public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
@@ -422,14 +464,12 @@ public final class FolderWriter {
                             fo.setFolderName(folder.getName());
                             fo.setModule(folder.getContentType().getModule());
                             fo.setType(folder.getType().getType());
-                            jsonPutter.put(additionalFolderField.getColumnName(), additionalFolderField.getValue(fo, serverSession));
+                            fo.setCreatedBy(folder.getCreatedBy());
+                            jsonPutter.put(aff.getColumnName(), aff.renderJSON(aff.getValue(fo, serverSession)));
                         }
                     };
                 } else {
-                    if (LOG.isWarnEnabled()) {
-                        LOG.warn("Unknown field: " + curCol, new Throwable());
-                    }
-                    ffw = UNKNOWN_FIELD_FFW;
+                    ffw = getPropertyByField(curCol, fieldSet);
                 }
             }
             ffws[i] = ffw;
@@ -438,12 +478,16 @@ public final class FolderWriter {
             final JSONArray jsonArray = new JSONArray();
             final JSONArrayPutter jsonPutter = new JSONArrayPutter();
             for (final UserizedFolder folder : folders) {
-                final JSONArray folderArray = new JSONArray();
-                jsonPutter.setJSONArray(folderArray);
-                for (final FolderFieldWriter ffw : ffws) {
-                    ffw.writeField(jsonPutter, folder);
+                try {
+                    final JSONArray folderArray = new JSONArray();
+                    jsonPutter.setJSONArray(folderArray);
+                    for (final FolderFieldWriter ffw : ffws) {
+                        ffw.writeField(jsonPutter, folder);
+                    }
+                    jsonArray.put(folderArray);
+                } catch (final NecessaryValueMissingException e) {
+                    LOG.warn(e.getMessage());
                 }
-                jsonArray.put(folderArray);
             }
             return jsonArray;
         } catch (final JSONException e) {
@@ -451,6 +495,25 @@ public final class FolderWriter {
         }
     }
 
+    private static List<FolderObject> turnIntoFolderObjects(final UserizedFolder[] folders) {
+        final List<FolderObject> retval = new ArrayList<FolderObject>(folders.length);
+        for (final UserizedFolder folder : folders) {
+            final FolderObject fo = new FolderObject();
+            final int numFolderId = getUnsignedInteger(folder.getID());
+            if (numFolderId < 0) {
+                fo.setFullName(folder.getID());
+            } else {
+                fo.setObjectID(numFolderId);
+            }
+            fo.setFolderName(folder.getName());
+            fo.setModule(folder.getContentType().getModule());
+            fo.setType(folder.getType().getType());
+            fo.setCreatedBy(folder.getCreatedBy());
+            retval.add(fo);
+        }
+        return retval;
+    }
+
     /**
      * Writes requested fields of given folders into a JSON array consisting of JSON arrays.
      * 
@@ -462,13 +525,11 @@ public final class FolderWriter {
     public static JSONArray writeMultiple2Array(final int[] fields, final Collection<UserizedFolder> folders) throws FolderException {
         final int[] cols = null == fields ? ALL_FIELDS : fields;
         final FolderFieldWriter[] ffws = new FolderFieldWriter[cols.length];
+        final TIntObjectMap<com.openexchange.folderstorage.FolderField> fieldSet = FolderFieldRegistry.getInstance().getFields();
         for (int i = 0; i < ffws.length; i++) {
             FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(cols[i]);
             if (null == ffw) {
-                if (LOG.isWarnEnabled()) {
-                    LOG.warn("Unknown field: " + cols[i], new Throwable());
-                }
-                ffw = UNKNOWN_FIELD_FFW;
+                ffw = getPropertyByField(cols[i], fieldSet);
             }
             ffws[i] = ffw;
         }
@@ -476,12 +537,16 @@ public final class FolderWriter {
             final JSONArray jsonArray = new JSONArray();
             final JSONArrayPutter jsonPutter = new JSONArrayPutter();
             for (final UserizedFolder folder : folders) {
-                final JSONArray folderArray = new JSONArray();
-                jsonPutter.setJSONArray(folderArray);
-                for (final FolderFieldWriter ffw : ffws) {
-                    ffw.writeField(jsonPutter, folder);
+                try {
+                    final JSONArray folderArray = new JSONArray();
+                    jsonPutter.setJSONArray(folderArray);
+                    for (final FolderFieldWriter ffw : ffws) {
+                        ffw.writeField(jsonPutter, folder);
+                    }
+                    jsonArray.put(folderArray);
+                } catch (final NecessaryValueMissingException e) {
+                    LOG.warn(e.getMessage());
                 }
-                jsonArray.put(folderArray);
             }
             return jsonArray;
         } catch (final JSONException e) {
@@ -497,16 +562,36 @@ public final class FolderWriter {
      * @return The JSON object carrying requested fields of given folder
      * @throws FolderException If writing JSON object fails
      */
-    public static JSONObject writeSingle2Object(final int[] fields, final UserizedFolder folder) throws FolderException {
-        final int[] cols = null == fields ? ALL_FIELDS : fields;
+    public static JSONObject writeSingle2Object(final int[] fields, final UserizedFolder folder, final ServerSession serverSession, final AdditionalFolderFieldList additionalFolderFieldList) throws FolderException {
+        final int[] cols = null == fields ? getAllFields(additionalFolderFieldList) : fields;
         final FolderFieldWriter[] ffws = new FolderFieldWriter[cols.length];
+        final TIntObjectMap<com.openexchange.folderstorage.FolderField> fieldSet = FolderFieldRegistry.getInstance().getFields();
         for (int i = 0; i < ffws.length; i++) {
-            FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(cols[i]);
+            final int curCol = cols[i];
+            FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(curCol);
             if (null == ffw) {
-                if (LOG.isWarnEnabled()) {
-                    LOG.warn("Unknown field: " + cols[i], new Throwable());
+                if (additionalFolderFieldList.knows(curCol)) {
+                    final AdditionalFolderField aff = additionalFolderFieldList.get(curCol);
+                    ffw = new FolderFieldWriter() {
+
+                        public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
+                            final FolderObject fo = new FolderObject();
+                            final int numFolderId = getUnsignedInteger(folder.getID());
+                            if (numFolderId < 0) {
+                                fo.setFullName(folder.getID());
+                            } else {
+                                fo.setObjectID(numFolderId);
+                            }
+                            fo.setFolderName(folder.getName());
+                            fo.setModule(folder.getContentType().getModule());
+                            fo.setType(folder.getType().getType());
+                            fo.setCreatedBy(folder.getCreatedBy());
+                            jsonPutter.put(aff.getColumnName(), aff.renderJSON(aff.getValue(fo, serverSession)));
+                        }
+                    };
+                } else {
+                    ffw = getPropertyByField(curCol, fieldSet);
                 }
-                ffw = UNKNOWN_FIELD_FFW;
             }
             ffws[i] = ffw;
         }
@@ -519,9 +604,18 @@ public final class FolderWriter {
             return jsonObject;
         } catch (final JSONException e) {
             throw FolderExceptionErrorMessage.JSON_ERROR.create(e, e.getMessage());
+        } catch (final NecessaryValueMissingException e) {
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
 
+    private static int[] getAllFields(final AdditionalFolderFieldList additionalFolderFieldList) {
+        final TIntList list = new TIntArrayList();
+        list.add(ALL_FIELDS);
+        list.add(additionalFolderFieldList.getKnownFields());
+        return list.toArray();
+    }
+
     /**
      * Writes requested fields of given folders into a JSON array consisting of JSON objects.
      * 
@@ -533,13 +627,11 @@ public final class FolderWriter {
     public static JSONArray writeMultiple2Object(final int[] fields, final UserizedFolder[] folders) throws FolderException {
         final int[] cols = null == fields ? ALL_FIELDS : fields;
         final FolderFieldWriter[] ffws = new FolderFieldWriter[cols.length];
+        final TIntObjectMap<com.openexchange.folderstorage.FolderField> fieldSet = FolderFieldRegistry.getInstance().getFields();
         for (int i = 0; i < ffws.length; i++) {
             FolderFieldWriter ffw = STATIC_WRITERS_MAP.get(cols[i]);
             if (null == ffw) {
-                if (LOG.isWarnEnabled()) {
-                    LOG.warn("Unknown field: " + cols[i], new Throwable());
-                }
-                ffw = UNKNOWN_FIELD_FFW;
+                ffw = getPropertyByField(cols[i], fieldSet);
             }
             ffws[i] = ffw;
         }
@@ -547,12 +639,16 @@ public final class FolderWriter {
             final JSONArray jsonArray = new JSONArray();
             final JSONObjectPutter jsonPutter = new JSONObjectPutter();
             for (final UserizedFolder folder : folders) {
-                final JSONObject folderObject = new JSONObject();
-                jsonPutter.setJSONObject(folderObject);
-                for (final FolderFieldWriter ffw : ffws) {
-                    ffw.writeField(jsonPutter, folder);
+                try {
+                    final JSONObject folderObject = new JSONObject();
+                    jsonPutter.setJSONObject(folderObject);
+                    for (final FolderFieldWriter ffw : ffws) {
+                        ffw.writeField(jsonPutter, folder);
+                    }
+                    jsonArray.put(folderObject);
+                } catch (final NecessaryValueMissingException e) {
+                    LOG.warn(e.getMessage());
                 }
-                jsonArray.put(folderObject);
             }
             return jsonArray;
         } catch (final JSONException e) {
@@ -564,8 +660,56 @@ public final class FolderWriter {
      * Helper methods
      */
 
+    private static final class PropertyFieldWriter implements FolderFieldWriter {
+
+        private final com.openexchange.folderstorage.FolderField field;
+
+        private final String name;
+
+        protected PropertyFieldWriter(final com.openexchange.folderstorage.FolderField field) {
+            super();
+            this.field = field;
+            final String name = field.getName();
+            this.name = null == name ? Integer.toString(field.getField()) : name;
+        }
+
+        public void writeField(final JSONValuePutter jsonPutter, final UserizedFolder folder) throws JSONException {
+            final FolderProperty property = folder.getProperties().get(field);
+            jsonPutter.put(name, null == property ? field.getDefaultValue() : property.getValue());
+        }
+
+    }
+
+    private static final ConcurrentTIntObjectHashMap<PropertyFieldWriter> PROPERTY_WRITERS =
+        new ConcurrentTIntObjectHashMap<PropertyFieldWriter>(4);
+
+    private static FolderFieldWriter getPropertyByField(final int field, final TIntObjectMap<com.openexchange.folderstorage.FolderField> fields) {
+        final com.openexchange.folderstorage.FolderField fieldNamePair = fields.get(field);
+        if (null == fieldNamePair) {
+            if (WARN) {
+                LOG.warn("Unknown field: " + field, new Throwable());
+            }
+            return UNKNOWN_FIELD_FFW;
+        }
+        PropertyFieldWriter pw = PROPERTY_WRITERS.get(field);
+        if (null == pw) {
+            final PropertyFieldWriter npw = new PropertyFieldWriter(fieldNamePair);
+            pw = PROPERTY_WRITERS.putIfAbsent(field, npw);
+            if (null == pw) {
+                pw = npw;
+            }
+        }
+        return pw;
+    }
+
+    /**
+     * The actual max permission that can be transfered in field 'bits' or JSON's permission object
+     */
+    private static final int MAX_PERMISSION = 64;
+
     private static final TIntIntHashMap MAPPING = new TIntIntHashMap(6) {
-        { //Unnamed Block.
+
+        { // Unnamed Block.
             put(Permission.MAX_PERMISSION, MAX_PERMISSION);
             put(MAX_PERMISSION, MAX_PERMISSION);
             put(0, 0);
@@ -584,15 +728,10 @@ public final class FolderWriter {
             perm.isAdmin());
     }
 
-    /**
-     * The actual max permission that can be transfered in field 'bits' or JSON's permission object
-     */
-    private static final int MAX_PERMISSION = 64;
-
     static int createPermissionBits(final int fp, final int rp, final int wp, final int dp, final boolean adminFlag) {
         int retval = 0;
         int i = 4;
-        retval += (adminFlag ? 1 : 0) << (i-- * 7)/*Number of bits to be shifted*/;
+        retval += (adminFlag ? 1 : 0) << (i-- * 7)/* Number of bits to be shifted */;
         retval += MAPPING.get(dp) << (i-- * 7);
         retval += MAPPING.get(wp) << (i-- * 7);
         retval += MAPPING.get(rp) << (i-- * 7);
